#!/usr/bin/perl

#
# time.cgi -- authored by paal@ekran.no, Wed Jan  5 08:49:13 CET 2000
# ---------------------------------------------------------------------

# Time Manager is a system that keeps track of how much time you have
# spent at work (or whatever). This system is ideal at workplaces which
# has a flexible time system.

# ---------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA02111-1307, USA.
# ---------------------------------------------------------------------

use Date::Manip;
use File::Copy;
use CGI qw(:cgi-lib);

# ---------------------------------------------------------------------
# the hostname, starting with http://, no trailing / 
$web_host='http://localhost';

# here's a relative path used in the forms.
$cgi_action='/cgi-bin/time/time.cgi';

# you should not alter the next line.
$cgi_url=$web_host.$cgi_action;

# comment out the language you don't want, if you comment out more than
# one language, the last language you comment out will be the default.
lang_eng(); # only english is currently supported.

# date_format. which is (day/month-year hour:minute:seconds)
# see the Date::Manip man pages if you want to change this.
$date_format="%d/%m-%Y %H:%M:%S";
$date_formatD="%d/%m";
$date_formatT="%H:%M:%S";

$screen_bgcolor="#ffc695";
$table_bgcolor="#efb685";
$text_hcolor='<font color="#000050">';
$text_red='<font color="#800000">';
$text_black='<font color="#000000">';
$text_color='#000000';

$std_table='<table border=1 cellpadding=2 bgcolor="'.
			$table_bgcolor.'" align="center">';
$std_tableL='<table border=1 cellpadding=2 bgcolor="'.$table_bgcolor.'">';

# mandatory workhours per week. the difference betwenn this and actual
# worked workhours are either overtime or time that the employee has to
# work inn.

$hours_a_day=7.5;
$hours_weekly=(5*$hours_a_day);

# ---------------------------------------------------------------------
# initialize -- nothing below this point should be altered! 

$VERSION="0.2.0";
$NAME="Time Manager";
$ENV{PATH}="/bin:/usr/bin:/usr/local/bin";
$LIVE_CODE=0; # zero if this is the official release
&Date_Init("Language=English","DateFormat=non-US");

&ReadParse(*data);

$uact=$data{"UACT"};
$uweek=$data{"UWEEK"};

if ($uact eq "login")
{
	$unam=$data{"UNAME"};
	$upwd=$data{"UPWD"};
}

# a list of legal UACT "commands" (must start and end with a whitespace)
$commands=" signon signoff stats delete del.exe week week.exe change change.exe change2.exe logout schedule schedule2 schedule3 ";

$dat=`date`; chomp $dat;

$entries=0;
$this_year=&UnixDate($dat,"%Y"); # this_year is current year YYYY

# --- user authentication handlers
if ($uact eq "logout") { logout(); }

$foo=0;
if ($uact eq "login")
{
	authenticate_setcookie($unam,$upwd);            # set a cookie
	$foo=1;
} else {
	print &PrintHeader;
	# try to grab the cookie user info. foo == 0 if cookie or input is bad.
	($foo,$unam,$upwd)=authenticate_parse_cookie();
}
if ($foo==0) { authenticate_user(); } # display the login screen

(@dblist)=read_datafile($unam);  # read the datafile
$entries=pop @dblist;

(@slist)=read_sfile($unam);  # read schedule file

if (!($uact)) # no command given, write menu screen
{
	$the_user=chk_user($unam,$upwd); # just grab user name
	head(1);
	menu_page();
	tail(2);
	exit 0;
}

head(); # print web header.

# ---------------------------------------------------------------------------
# check for errors

$err=0;
if (!($the_user=chk_user($unam,$upwd))) # check user authentication
{
    print $l_auth_failed."<br>\n";
	$err++; 
}

$foo=" ".$uact." ";
$_=$commands;
if (!(m/$foo/i ))
{
    print $l_action_n_spec."<br>\n";
    $err++;
}

# check previous entry -- we can't login/-out twice in a row
$line=$dblist[$entries];
($id,$action,$date)=split /;/,$line;
if (($uact eq "signoff") && ($id/2==(int($id/2)))) {
   print "<br>".$l_bad_seq."<br>\n"; $err++;
}
if (($uact eq "signon") && ($id/2!=(int($id/2)))) {
   print "<br>".$l_bad_seq."<br>\n"; $err++;
}

# ---------------------
# debug section
# foreach $line (@dblist) { print "->".$line."<-<br>\n"; } 

# ---------------------------------------------------------------------------
# no errors found lets get goin'

print "<br>\n";
if ($err==0) {
    if (($uact eq "signon") || ($uact eq "signoff")) { # store signon/out entry
       add_entry($unam,$uact);
    } 
    if ($uact eq "stats") { # show statistics
       show_statistics($unam);
    }
    if ($uact eq "delete") {  # select deletion
       prep_delete($unam);
    }
    if ($uact eq "del.exe") {  # delete entries
       exec_delete($unam);
    }
    if ($uact eq "week") { # week select
       select_week();
    }
    if ($uact eq "week.exe") { # week perform
       mkweek2($unam,$uweek);
    }
    if ($uact eq "change") { # change entry selection
       prep_change($unam);
    }
    if ($uact eq "change.exe") { # change the selected selections
       exec_change($unam);
    }
    if ($uact eq "change2.exe") { # change execute
       exec_change2($unam);
    }
    if ($uact eq "schedule") { # schedule select month
       exec_schedule($unam);
    }
    if ($uact eq "schedule2") { # schedule perform reschedule
       exec_schedule2($unam);
	}
    if ($uact eq "schedule3") { # schedule execute
       exec_schedule3($unam);
	}
}
tail(0);
exit 0;


# ---------------------------------------------------------------------------
# subroutines
# ---------------------------------------------------------------------------

# ---------------------------------------------------------------------------
# COOKIE AUTHENTICATION

# authenticate_parse_cookie reads and parses the cookie environment variable\
# then returns username and password if it finds any.

sub authenticate_parse_cookie() 
{
	my $ck;
	my $key;
	my $value;
	my $kv;
	my $unam;
	my $upwd;

	$ck=0;
	@cookies=split (/;\s/,$ENV{'HTTP_COOKIE'}); # splits the cookies add table
	foreach $kv (@cookies)
	{
		($key,$value)=split (/=/,$kv);
		if ($key eq "user") {$unam=$value;}
		if ($key eq "pwd") {$upwd=$value;}
	}

	if (($unam) && ($upwd)) { $ck=1; }
	return ($ck,$unam,$upwd);
}

sub authenticate_user() # writes the html login screen.
{
	head();
	login_page();
	tail(1);
	exit 0;
}

sub authenticate_setcookie()
{
	my ($user,$pwd)=@_;
	
	print "Content-Type: text/html\n"; 
	if ($the_user=chk_user($user,$pwd))
	{
		print "Set-Cookie: user=".$user.";","\n";
		print "Set-Cookie: pwd=".$pwd.";","\n\n";
	
		$uact=""; # this forces a printout of the menu screen	
	} else {
		print "\n";
		head();
		print '<center>User failed to authenticate.<br></center>'."\n";
		tail(1);
		exit 0;
	}	
	return ($user,$pwd);
}

sub logout() # log user off the system.
{
	print "Content-Type: text/html\n"; 
	print "Set-Cookie: user=;","\n";
	print "Set-Cookie: pwd=;","\n\n";
	head();
	print '<center><br><b>'.$l_logedout.'</b><br></center>'."\n";
	tail(1);
	exit 0;
}

# ---------------------------------------------------------------------------
# DISPLAY LOGIN SCREEN

sub login_page() 
{
	print '<form action="'.$cgi_action.'" method="POST">
			'.$std_table.'
			<tr>
  				<th colspan=3>'.$text_hcolor.'AUTHENTICATE USER</font></th>
			</tr>
			<tr>
  				<td>Username:</td>
  				<td align=right colspan=2>
					<input type="text" name="UNAME" size=8></td>
			</tr>
			<tr>
  				<td>Password:</td>
  				<td align=right colspan=2>
					<input type="password" name="UPWD" size=8></td>
			</tr>
			<tr>
  				<td align=right colspan=3>
  				<input type="submit" name="UACT" value="login">
  				<input type="reset" value="clear"></td>
			</tr>
			</table></form>';
}


# ---------------------------------------------------------------------------
# DISPLAY MENU SCREEN

sub menu_page()
{

print '<form action="'.$cgi_action.'" method="POST">
		'.$std_table.'
		<tr>
  			<th colspan=3>'.$text_hcolor.'WORK HOURS REGISTRATION</font></th>
		</tr>
		<tr>
  			<td>Sign on</td>
			<td align=right colspan=2>
				<input type="submit" name="UACT" value="signon"></td>
		</tr>
		<tr>
  			<td>Sign off</td>
  			<td align=right colspan=2>
      			<input type="submit" name="UACT" value="signoff"></td>
		</tr>
		<tr>
			<td colspan=3>&nbsp;</td>
		</tr>
		<tr>
  			<td>Week report</td>
  			<td align=right colspan=2>
      		<input type="submit" name="UACT" value="week"></td>
		</tr>
		<tr>
  			<td>General statistics</td>
  			<td align=right colspan=2>
				<input type="submit" name="UACT" value="stats"></td>
		</tr>
		<tr>
  			<td>Delete entry</td>
  			<td align=right colspan=2>
				<input type="submit" name="UACT" value="delete"></td>
		</tr>
		<tr>
  			<td>Alter entry</td>
  			<td align=right colspan=2>
				<input type="submit" name="UACT" value="change"></td>
		</tr>
		<tr>
  			<td>Scheduled work hours</td>
  			<td align=right colspan=2>
				<input type="submit" name="UACT" value="schedule"></td>
		</tr>
		<tr>
			<td colspan=3>&nbsp;</td>
		</tr>
		<tr>
  			<td>Logout from Time Manager</td>
  			<td align=right colspan=2>
				<input type="submit" name="UACT" value="logout"></td>
		</tr>

		</table>
		</form>
	';
}

# ---------------------------------------------------------------------------
# ADD ENTRY

sub add_entry()
{
	($unam,$uact)=@_;
	if ($uact eq "signon") { $action="inn"; } else { $action="out"; }

   	open DTA,">>data/".$unam.".".$this_year.".dta" ||
    	print "can't open datafile for writing...<br>\n";
	print DTA ($next_entry+1).";".$action.";".&ParseDate($dat)."\n";
	close DTA;
	print $uact." ".$l_confirm_action." at ".&UnixDate($dat,$date_format).
			"<br>\n";
}


# ---------------------------------------------------------------------------
# ALTER ENTRY SUBS

sub prep_change()
{
	($unam)=@_;
	print '<form action="'.$cgi_action.'" method="POST">';
	print '<input type="submit" name="change.exe" value="change">';
	print '<input type="reset" value="reset">';
	print $std_tableL."\n";
	print '<tr><th colspan=3>'.$text_hcolor.$l_alter_select_entry.
			'</th></tr>';
	print '<tr><td>&nbsp;</td><th>'.
				$l_from.'</th><th>'.$l_to.'</td></tr>'."\n"; 
	$inn="empty"; $out="empty"; $cnt=0; 
	foreach $line (@dblist)
	{
		($id,$act,$date)=split /;/,$line;
		if ($inn eq "empty")
			{ $inn=$date; $ice=$id } else { $out=$date; }
		if ($out ne "empty")
		{
			if ($out eq "nempty") { $out=$l_still_online; }
			print '<td>
					<input type="checkbox" name="U'.$ice.'" value="unset">
					</td>
				   <td>'.&UnixDate($inn,$date_format).'</td><td>';
			if ($out eq $l_still_online)
			{
				print $l_still_online;
			} else {
				print &UnixDate($out,$date_format);
			}
			print '</td></tr>'."\n";
			$inn="empty"; $out="empty"; $cnt++;
		} 
	}
	print '</table>'."\n";
	print '<input type="hidden" name="UACT" value="change.exe">';
	print '<input type="submit" name="change.exe" value="change">';
	print '<input type="reset" value="reset">';
}

sub exec_change()
{
	($unam)=@_;

	print '<form action="'.$cgi_action.'" method="POST">';
	print '<input type="submit" value="change">';
	print '<input type="reset" value="reset">';

	print $std_tableL."\n";
	print '<tr><th colspan=7>'.$text_hcolor.$l_alter_changeent.'</th></tr>';
	print '<tr><th>original</th>
               <th>day</th>
               <th>mon</th>
               <th>&nbsp;</th>
               <th>hrs</th>
               <th>min</th>
               <th>sec</th></tr>'."\n";

	foreach $line (@dblist)
	{
		($id,$act,$date)=split /;/,$line;
		if ((($data{"U".$id} eq "unset") || 
			($data{"U".($id-1)} eq "unset")) && ($date ne "nempty"))
		{
			print '<tr><td>'.&UnixDate($date,$date_format).'</td>';
			print '  <td><input type="text" size=2 name="dayU'.$id.'" value="'.&UnixDate($date,"%d").'"></td>';
			print '  <td><input type="text" size=2 name="monU'.$id.'" value="'.&UnixDate($date,"%m").'"></td>';
			print '  <td>&nbsp;</td>';

			print '  <td><input type="text" size=2 name="hrsU'.$id.'" value="'.&UnixDate($date,"%H").'"></td>';
			print '  <td><input type="text" size=2 name="minU'.$id.'" value="'.&UnixDate($date,"%M").'"></td>';
			print '  <td><input type="text" size=2 name="secU'.$id.'" value="'.&UnixDate($date,"%S").'"></td>';
			print '</tr>'."\n";
			if (($id/2)==(int($id/2)))
				{ print '<tr><td></td></tr>'; }
		}
	}
    print '</table>'."\n";
    print '<input type="hidden" name="UACT" value="change2.exe">';
    print '<input type="submit" value="change">';
    print '<input type="reset" value="reset">';
    print '</form>';
}

sub exec_change2() { # perform the actual change
	($unam)=@_;

	my $idn; # id next.
	my $thise; # a regular tuppel counter
	my ($md,$mm,$mh,$mi,$ms,$ctl);
	my ($prev,$next,$thise,$nowrite);

	print "<b>".$l_alter_entries."</b><p>";
	print "<ul>";

	$thise=0; $nowrite=1;
	foreach $line (@dblist)
	{
		($id,$act,$date)=split /;/,$line;
		$ctl=0;
		$thise++;
		if ($data{"dayU".$id})
		{
			($prev,$next)=getnextprev($id); # get the next and previous entry
			$mm=$data{"monU".$id};
			$md=$data{"dayU".$id};
			$mh=$data{"hrsU".$id};
			$mi=$data{"minU".$id};
			$ms=$data{"secU".$id};

			$dm_new=$this_year.$mm.$md.$mh.":".$mi.":".$ms;
			$ctl=0;
			
			if ($dm_new ne $date) # check if date has been alter before ctrl.
			{
				if ($data{"dayU".($id+1)}) # next record is also altered
				{
					$idn=($id+1);
					$next=$this_year.
							$data{"monU".$idn}.
							$data{"dayU".$idn}.
							$data{"hrsU".$idn}.":".
							$data{"minU".$idn}.":".
							$data{"secU".$idn};
					if (!(&ParseDate($next)))
					{
						print $l_alter_nextillegal.".<br>";
						$ctl=+1;
					}
				}

				if (($dm_new lt $prev) && (&ParseDate($dm_new)) &&
					(&ParseDate($prev)))
				{
					print $l_alter_eprealt."<br>";
					print '<b>'.$l_alter_current.':</b> ('.
&UnixDate($date,$date_format).")<b> ".$l_alter_new.':</b> ('.
&UnixDate($dm_new,$date_format).")<b> ".$l_alter_prev.':</b> ('.
&UnixDate($prev,$date_format).")<p>";
					$ctl=1;
				}

				if (($dm_new gt $next) && (&ParseDate($dm_new))
					&& (&ParseDate($next)))
				{
					print $l_alter_enextalt."<br>";
					print	'<b>'.$l_alter_current.':</b> ('.
&UnixDate($date,$date_format).")<b> ".$l_alter_new.':</b> ('.
&UnixDate($dm_new,$date_format).")<b> ".$l_alter_next.':</b> ('.
&UnixDate($next,$date_format).")<p>";
					$ctl=+1;
				}

				if ($dm_new gt &ParseDate(`date`)) # can't alter to future
				{
					print $l_alter_futurealt."<br>";
					print '<b> '.$l_alter_current.'</b> ('.
&UnixDate(&ParseDate(`date`),$date_format).') <b>'.$l_alter_new.'</b> ('.
&UnixDate($dm_new,$date_format).')<br>'."\n";	
					$ctl=+1;
				}

				if (!(&ParseDate($dm_new))) # illegal date
				{
					print $l_alter_illegaldate." (".
					$md."/".$mm."-".$this_year." ".$mh.":".$mi.":".$ms.")<br>";
					$ctl=+1;
				}
			}

			print &UnixDate($date,$date_format)." -> ".
				  &UnixDate($dm_new,$date_format)." "; # write date onto scrn.

			if ($dm_new eq $date) { # nothing has been altered.
				print "--> ".$l_alter_noch.".";
				$ctl=0;
			}
		
			if ($ctl!=0) # something went wrong
			{
				print $l_alter_failed.".";
				$nowrite=2;
			}

			if (($ctl==0) && ($dm_new ne $date)) # success!
			{
				$foo=$id.";".$act.";".$dm_new;
				$dblist[($thise-1)]=$foo;
				print "--> ".$l_alter_ch.".";
				if ($nowrite ne 2) { $nowrite=0; }
			}
			print "<br><p>\n";
    	}
	}
	print "</ul>";
	if ($nowrite==0) # write how it went.
	{
		alter_file();
		print '<b>Changes has been saved.</b>';
	} elsif ($nowrite==1) {
		print '<b>No changes were made.</b>';
	} elsif ($nowrite==2) {
		print '<b>NOTE: One or more errors were found.
						No changes were saved!</b>';
	}
	print '<br>';
}

sub alter_file()
{
	open DTA,">".$dtafile; # write new table to datafile.
	foreach $lineb (@dblist)
 	{
		($foo,$foo,$datex)=split /;/,$lineb;
		if ($datex ne "nempty")
		{
			print DTA $lineb."\n"; # a new line
		}
	}
	close DTA;
}

sub getnextprev() # returns next and previous entry in the table
{
	my ($ida)=@_;

	my ($idb,$chk,$linex,$timex,$prev,$next);

	$chk=0;
	foreach $linex (@dblist)
	{
		($idb,$foo,$timex)=split /;/,$linex;
		if ($chk==1) { $next=$timex; $chk=2;} # grab next in next turn
		if (($ida eq $idb) && ($chk==0)) { $chk=1; }
		if ($chk==0) { $prev=$timex; } # grab previous while not found
	}
	return ($prev,$next);
}

# ---------------------------------------------------------------------------
# DELETE
 
# prepare to delete entry 
sub prep_delete()
{
	($unam)=@_;
	print '<form action="'.$cgi_action.'" method="POST">';
	print $std_tableL."\n";
	print '<tr><th colspan=3>'.$text_hcolor.$l_select_del_title.'</th></tr>
			<tr><td>&nbsp;</td><th>'.$l_from.'</th><th>'.
				$l_to.'</td></tr>'."\n"; 
	print '<input type="submit" value="delete">
			<input type="reset" value="reset">';
	$inn="empty"; $out="empty"; $cnt=0; 
	foreach $line (@dblist)
	{
		($id,$act,$date)=split /;/,$line;
		if ($inn eq "empty")
			{ $inn=$date; } else { $out=$date; }
		if ($out ne "empty") {
			if ($out eq "nempty") { $out=$l_still_online; }
			print '<td><input type="checkbox" name="U'.
					   ($id-1).'" value="unset"></td>
				   <td>'.&UnixDate($inn,$date_format).'</td><td>';
			if ($out eq $l_still_online)
			{
				print $l_still_online;
			} else {
				print &UnixDate($out,$date_format);
			}
		    print '</td></tr>'."\n";
			$inn="empty"; $out="empty"; $cnt++;
		} 
	}
	print '</table>'."\n";
	print '<input type="hidden" name="UACT" value="del.exe">';
	print '<input type="submit" value="delete">';
	print '<input type="reset" value="reset">';
}

# execute the deletion
sub exec_delete() {
	my $lcnt;

	foreach $line (@dblist) { # pick the ones not set for deletion
		($id,$act,$date)=split /;/,$line;
		if (($data{"U".$id} eq "unset") || ($data{"U".($id-1)} eq "unset"))
		{
			push @deleted,$line; # for information purposes only
		} else {
			push @newlst,$line;  # these we wanna keep
		}
     	$prev=$line;
  	}

	$bakfile="date/".$unam.".".$this_year.".bak";
	copy ($dtafile,$bakfile); # Make a backup file

	open DTA,">".$dtafile; # write new datafile
	foreach $line (@newlst)
	{
		if (substr($line,length($line)-6,6) ne "nempty")
		{
   		 	print DTA $line."\n"; 
    	}
  	}
  	close DTA;

  	print "<b>".$l_deleted_lines."</b><p><ul>\n";
  	$cnt=0; 
	$lcnt=0;
  	
	foreach $line (@deleted)
	{
   		($foo,$foo,$line)=split /;/,$line;
       	if (($cnt/2) != (int($cnt/2)))
		{
			print &UnixDate($prev,$date_format);
			if ($line ne "nempty")
			{
				print " to ".&UnixDate($line,$date_format);
			} 
			print "<br>\n";
            $lcnt=1;
		}
       	$cnt++;
       	$prev=$line;
  	}
	print '</ul>';
  	if ($lcnt==0)
	{
     	print "<dd>".$l_nodeleted_lines."<br>";
  	}
}




# ---------------------------------------------------------------------------
# TIME SCHEDULER 

# ------------------------------------------
# time scheduler select month to alternation
sub exec_schedule() 
{
	($unam)=@_;

	my $cnt;
	my $bolt='<input type="radio" name="UMON" value="';

	print '<form action="'.$cgi_action.'" method="POST">';
	print $std_tableL."\n";
	print '<tr><th colspan=2>'.$text_hcolor.$l_select_month_sched.'
			</th></tr><tr>
			<td width=50%>';

	for ($cnt=0; $cnt<12; $cnt++)
	{
		if ($cnt<6) { print $bolt.$cnt.'">'.$year_name[$cnt].'<br>'; }
		if ($cnt==6) { print '</td><td align=right>'; }
		if ($cnt>=6) { print $year_name[$cnt].$bolt.$cnt.'"><br>'; }
	}

	print '</td><tr><td colspan="2" align="center">
			<input type="hidden" name="UACT" value="schedule2">
			<input type="submit" value="Ok"></td></tr>
			</table><br>';
}

# ---------------------------------------
# displays month and schedule information
sub exec_schedule2()
{
	($unam)=@_;
	my $dow; # date of week
	my $doy; # day of year
	my $dim; # days in month
	my $mon=$data{"UMON"};

	my $s,$h,$d;

	print '<form action="'.$cgi_action.'" method="POST">';
	print $std_tableL.'<tr><th colspan=8>'.$l_work_schedule_for.' '.
				$year_name[$mon].' '.$this_year.'</th></tr>';

	$mon++; # date::manip months starts at 1
	$dim=&Date_DaysInMonth($mon,$this_year);

	print '<tr><th>'.lc($l_week_no).'</th>';
	foreach $day (@week_name)
	{
		print '<th>'.$day.'</th>';
	}
	print '</tr>'."\n";

	for ($cnt=1; $cnt<=$dim; $cnt++)
	{
		$dow=&Date_DayOfWeek($mon,$cnt,$this_year);
		if (($dow==1) || ($cnt==1))
		{
			print '<tr><td>'.
					(&Date_WeekOfYear($mon,$cnt,$this_year,1)+1).'</td>';
			if (($cnt==1) && ($dow!=1))  # week doesn't start on a monday
			{
				for ($cnt2=1; $cnt2<=$dow-1; $cnt2++)
				{
					print '<td>&nbsp;</td>';
				}
			}
		}
		$doy=&Date_DayOfYear($mon,$cnt,$this_year); # this day in this year
		($d,$h,$s)=split /;/,$slist[$doy-1]; # schedules list of this day

		print '<td align="center">
		 		<b>'.$cnt.'/'.$mon.'</b>'.
		' - <input type="text" value="'.$h.'" name="UH'.$doy.'" size=4><br>
		 <select name="US'.$doy.'">
	 	 <option value="'.$s.'"> '.$s;
		if ($s ne "W") { print '<option value="W"> W'; } 
		if ($s ne "N") { print '<option value="N"> N'; } 
		if ($s ne "H") { print '<option value="H"> H'; } 
		if ($s ne "S") { print '<option value="S"> S'; } 
		if ($s ne "V") { print '<option value="V"> V'; }
		if ($s ne "U") { print '<option value="U"> U'; }
		print '</select></td>';
		if (($cnt==$dim) && ($dow ne 7))
		{
			print '<td colspan="'.(7-$dow).'">&nbsp;</td>';
		}
		if (&Date_DayOfWeek($mon,$cnt,$this_year)==7) { print '</tr>'."\n"; } 
	}

	print'<tr><td colspan=8 align=center>
			Abbreviations: W=Workday, N=Non-Workday (Weekend),
			H=National Holliday, S=Sick, V=Vaccation and U=Unspecified.<p>
			</td></tr>';
	print '<tr><td colspan=8 align=right>
			<input type="hidden" name="UACT" value="schedule3">
			<input type="submit" value="Ok">
			<input type="reset" value="Reset">
			</table></form>';
}

# ----------------------------
# execute schedule alternation
sub exec_schedule3()
{
	my $cnt;
	my $s,$h,$d;
	my $m,$d;
	my $er;

	$er=0;
	print '<b>Altering work schedule.</b><p><ul>';
	for ($cnt=1; $cnt<367; $cnt++)
	{
		if (($data{"UH".$cnt}) || ($data{"US".$cnt}))
		{
			($d,$h,$s)=split /;/,$slist[$cnt-1];
			if (($h ne $data{"UH".$cnt}) || ($s ne $data{"US".$cnt}))
			{
				($foo,$m,$d,$foo,$foo,$foo)=&Date_NthDayOfYear($this_year,$cnt);
				print 'Date: '.$d.'/'.$m.' was '.$h.':'.$s.' -> '.
						$data{"UH".$cnt}.':'.$data{"US".$cnt};

				if (($data{"UH".$cnt}<0) || ($data{"UH".$cnt}>24)) {
					print ' <b>Work hours out of range (0-24)!</b>';
					$er=2;
				} else { # no error, I hope.
					$slist[$cnt-1]=$cnt.";".$data{"UH".$cnt}.";".
											$data{"US".$cnt};
					if ($er!=2) {$er=1;}
				}
				print "<br>";
			}
		}	
	}
	print '</ul>';	
	print "\n";
	if ($er==2)
	{
		print '<b>Errors occoured. Data not saved!</b><br>';
	} elsif ($er==1) {
		print '<b>New schedule saved!</b><br>';
		exec_schedule_save();
	} else {
		print '<b>No change. Nothing done.</b><br>';
	}
	print "\n";
}

# -------------------------------
# save new schedule back to disc.
sub exec_schedule_save()
{
    my $dty=&Date_DaysInYear($this_year);
	my $cnt;

	$sfile="data/".$unam.".".$this_year.".schedule.dta";
	open DTA,">".$sfile;
    for ($cnt=1; $cnt<=$dty; $cnt++)
    {
		print DTA $slist[$cnt-1]."\n";
	}
	close DTA;
}



# ---------------------------------------------------------------------------
# WEEKLY REPORTS 

sub select_week() {
	print "<ul>";
	foreach $line (@dblist)
	{
		($id,$action,$date)=split /;/,$line;
		$wk=&UnixDate($date,"%W");
		if (($action eq "inn") && (0==chk_weeklst($wk))) {
        	push @av_week,$wk;
     	}
  	} 

  	print '<form action="'.$cgi_action.'" method="POST">';
	print $std_tableL.'
  		  <tr><th colspan=4>'.$text_hcolor.$l_select_week_display.'</th></tr>
  		  <tr><td>&nbsp;</td><th>'.$l_week_no.'</th><th>'.$l_start.'</th>
		  <th>'.$l_end.'</th></tr>';

	foreach $line (@av_week)
	{
		$from=&UnixDate(&ParseDate("monday week ".$line." of ".
						$this_year),"%d/%m");
 		$to=&UnixDate(&ParseDate("sunday week ".$line." of ".
						$this_year),"%d/%m");

		print '<tr><td>
				<input type="radio" name="UWEEK" value="'.$line.'"></td>
				<td align=center>'.$line.'</td>
				<td>'.$from.'</td><td>'.$to."</td></tr>"; 
  	}

	print '<tr><td colspan=4 align=right>';
	print '<input type="hidden" name="UACT" value="week.exe">';
	print '<input type="submit" value="show">';
	print '<input type="reset" value="reset">';
	print '</table>';
	print "<\/ul><br>";
 
}

# -----------------------------------------------------------------------
# check if a list has already been pushed onto the "available week list".
sub chk_weeklst()
{
	my ($wno)=@_;
	foreach $l (@av_week)
	{
		if ($l eq $wno) { return 1; } 
	}
	return 0;
}

# ---------------------------------
# this will be the next gen. mkweek
sub mkweek2()
{
	($unam,$uweek)=@_;

	my	@heweek; # hours expected (scheduled) a week
	my	@haweek; # hours actually worked a week
	my 	$coll;
	my	$date,$dateI,$dateO;
	my 	$delta;
	my	$tt,$tm,$ts; # total hour, minute, second

	# calculate how much is worked each week
	# 
	
	for ($cnt=1; $cnt<=8; $cnt++) # initalize week variables
	{
		push @day,"";
		push @dayH,0;
		push @dayM,0;
		push @dayS,0;
		push @dayC,0; # scheduled this day
	}
	for ($cnt=1; $cnt<=$uweek; $cnt++) # initialize weeks summary vars.
	{
		push @heweek,0;
		push @haweek,0;
	}

	foreach $line (@slist) # traverse schedule list.
	{
		($id,$hrs,$type)=split /;/,$line;
		($foo,$m,$d,$foo,$foo)=&Date_NthDayOfYear($this_year,$id);
		$woy=&Date_WeekOfYear($m,$d,$this_year,1);
		$heweek[$woy]=$heweek[$woy]+$hrs;
	}

	$dateI="UNSET"; $dateO="UNSET";
	$tt=0; $tm=0; $ts=0;
	
	foreach $line (@dblist)	
	{
		($id,$action,$date)=split /;/,$line;

		if ($date eq "nempty") { $date=&ParseDate($dat); } # i.e stillon

		$m=&UnixDate($date,"%m");
		$d=&UnixDate($date,"%d");
		$y=&UnixDate($date,"%Y");
		$wkno=&Date_WeekOfYear($m,$d,$y,1);
		if (($wkno<=$uweek) || (($action eq "out") && ($dateI ne "UNSET")))
		{
			if ($action eq "inn")
			{
				$dateI=$date;
			} else {
				$dateO=$date;

            	$delta=&DateCalc($dateO,$dateI,\$erro); # time spent
            	($foo,$foo,$uw,$ud,$ut,$um,$us)=split /:/,$delta;
				
            	while ($uw!=0) { $uw--; $ud=$ud+7; }  # long workhours
            	while ($ud!=0) { $ud--; $ut=$ut+24; } # very long workhours
				# new we should have hours spent on this session in(ut,um,us)
	
				$hours=int ($ut+(1/60*$um)+(1/3600*$us));
				$heweek[$wkno]=$heweek[$wkno]-$hours;

				if ($wkno == $uweek) # this is in selected week. count. -----
				{
					$dow=&Date_DayOfWeek($m,$d,$y);
					if ($day[$dow] ne "") { $day[$dow]=$day[$dow]."<br>";}
					$day[$dow]=$day[$dow].
										&UnixDate($dateI,$date_formatT)."<br>".
										&UnixDate($dateO,$date_formatT)."<br>";
					$tt=$tt+$ut; # collected this week!
					$tm=$tm+$um;
					$ts=$ts+$us;

					$dayH[$dow]=$dayH[$dow]+$ut;
					$dayM[$dow]=$dayM[$dow]+$um;
					$dayS[$dow]=$dayS[$dow]+$us;

					# now handle minute and second overflow.
					if ($dayS[$dow]>=60) {
						$dayM[$dow]++;
						$dayS[$dow]=$dayS[$dow]-60;
					}
					if ($dayM[$dow]>=60) {
						$dayH[$dow]++;
						$dayM[$dow]=$dayM[$dow]-60;
					}
					
				}

			}
		} else {
			if ($action eq "inn") { $dateI="UNSET"; } # ignore next too.
		}
	}
	$stime=0; # scheduled time, incl. this week
	$sptime=0; # scheduled time, excl. this week
	for ($cnt=1; $cnt<=$uweek; $cnt++)
	{
		$sptime=$stime;
		$stime=$stime+$heweek[$cnt];
	}
	$stime=-$stime;
	$sptime=-$sptime;

	# now all the data is gathered and we need to do a printout

	print $std_tableL;
	print '<tr><th colspan=7>'.$text_hcolor.$l_work_stat_week." ".
			($uweek).' for '.$the_user.'</th></tr>';
	print '<tr><th>day</th><th>date</th><th>from/to</th><th>hours</th>
			<th>minutes</th><th>seconds</th><th>scheduled</th></tr>';
	print '<tr><td colspan="6" align="left">&nbsp;</td><th align=right>';

	if ($sptime ge 0) { print $text_black; } else { print $text_red; }
	print $sptime;

	print '</font></th></tr>';
	
	for ($cnt=1; $cnt<=7; $cnt++)
	{

		print '<tr>';
		print '<td>'.$week_name[$cnt-1].'</td>';
		print '<td>'.&UnixDate(&ParseDate($week_name[$cnt-1]." week ".
								$uweek." ".$this_year),$date_formatD).'</td>';
		if ($day[$cnt]) 
		{
			print '<td>'.$day[$cnt].'</td>';
		} else {
			$foo=cempty($cnt);
			if (($day[$cnt-1] ne "") || ($cnt==1))
			{	
				print '<td rowspan="'.$foo.'">&nbsp;</td>';
			}
		}
		print '<td align="right">'.$dayH[$cnt].'</td>';
		print '<td align="right">'.$dayM[$cnt].'</td>';
		print '<td align="right">'.$dayS[$cnt].'</td>';

		($foo,$dayC[$cnt],$foo)=split /;/,$slist[&UnixDate(&ParseDate($week_name[$cnt-1]." week ".$uweek." ".$this_year),"%j")-1];
		print '<td align=right>'.$dayC[$cnt].'</td>';
		print '</tr>';
	}

	print '<tr><td colspan="3">total</td>';
	print '<td align="right">'.$tt.'</td>
			<td align="right">'.$tm.'</td>
			<td align="right">'.$ts.'</td>';

	print '<td align="right">';
	if ($stime<0) { print $text_red; } else { print $text_black; }
	print '<b>'.$stime.'</b></td></tr>';

	print '</table>';
}

sub cempty() # counts empty cells below an empty cell (to merge rowspan)
{
	($cnt)=@_;
	my $cntx;

	for ($cntx=$cnt; ($cntx<=7) && ($day[$cntx] eq ""); $cntx++) {}
	return ($cntx-$cnt);	
}



# ----------------------------------------------------------------------------
# display user statistics

sub show_statistics() {
   ($unam)=@_;

   my $uday=0; my $utim=0; my $umin=0; my $usec=0;
   my @wkt=(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
            0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
   my @wkm=(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
            0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
   my @wks=(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
            0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
   @mkt=(0,0,0,0,0,0,0,0,0,0,0,0,0,0); @mkm=(0,0,0,0,0,0,0,0,0,0,0,0,0,0);
   @mks=(0,0,0,0,0,0,0,0,0,0,0,0,0,0); # Month[hour.minute.sec] count
   @dows=(0,0,0,0,0,0,0,0,0,0,0,0,0,0); @dowm=(0,0,0,0,0,0,0,0,0,0,0,0,0,0);
   @dowt=(0,0,0,0,0,0,0,0,0,0,0,0,0,0); # Day of week [hour.minute.sec.] count


# ----------------------------------
# read and analyze data from time db

   $date1="empty"; $date2="empty";
   foreach $line (@dblist) {
       ($id,$action,$date)=split /;/,$line;
       if ($date1 eq "empty") {$date1=$date;} else {$date2=$date;}
       if ($date2 ne "empty") {
          if ($date2 eq "nempty") { # use current datestamp if still online
             $date2=$dat; $stillon=1;
          } else {$stillon=0;}

          $delta=&DateCalc($date2,$date1,\$erro); # time spent
          ($foo,$foo,$uw,$ud,$ut,$um,$us)=split /:/,$delta;
          while ($uw!=0) { $uw--; $ud=$ud+7; }
          while ($ud!=0) { $ud--; $ut=$ut+24; }

          ($y)=&UnixDate($date1,"%Y"); # find which week we are in.
          ($m)=&UnixDate($date1,"%m");
          ($d)=&UnixDate($date1,"%d");
          $wkno=&Date_WeekOfYear($m,$d,$y,1); # now we know what week we have
          $dow=&Date_DayOfWeek($m,$d,$y);     # check which day of week it is

       # add statistics to the misc. tables
          ($wkt[$wkno],$wkm[$wkno],$wks[$wkno])=
                addtms($wkt[$wkno],$wkm[$wkno],$wks[$wkno],$ut,$um,$us);

          ($mkt[$m],$mkm[$m],$mks[$m])=
                addtms($mkt[$m],$mkm[$m],$mks[$m],$ut,$um,$us);
          ($dowt[$dow],$dowm[$dow],$dows[$dow])=
                addtms($dowt[$dow],$dowm[$dow],$dows[$dow],$ut,$um,$us);
          ($utim,$umin,$usec)=addtms($utim,$umin,$usec,$ut,$um,$us);
          while ($utim>=24) {
             $uday++; $utim=$utim-24;
          }
          if ($stillon==1) { # user is still online, print dummy at logof field
              push @list,"<tr><td align=right>".&UnixDate($date1,"%d/%m-%Y %H:%M:%S")."<\/td><td align=right>".$l_still_online."</td><td align=right>".$ut."<\/td><td align=right>".$um."<\/td><td align=right>".$us."<\/td>\n";
          } else {
              push @list,"<tr><td align=right>".&UnixDate($date1,"%d/%m-%Y %H:%M:%S")."<\/td><td align=right>".&UnixDate($date2,"%d/%m-%Y %H:%M:%S")."</td><td align=right>".$ut."<\/td><td align=right>".$um."<\/td><td align=right>".$us."<\/td>\n";
          }
          $date1="empty"; $date2="empty";
       }
   }

# ---------------------------
# sum each weeks' total hours

   for ($cnt=1; $cnt<54; $cnt++) {
      # summarize work each week
      if (($wks[$cnt]!=0) || ($wkm[$cnt]!=0) || ($wkt[$cnt]!=0)) {
        push @week,"<tr><td align=right>".$cnt."<\/td><td align=right>".$wkt[$cnt]."<\/td><td align=right>".$wkm[$cnt]."<\/td><td align=right>".$wks[$cnt]."<\/td><\/tr>\n";
      } 
      # summarize work each month
      if (($cnt<13)&&(($mks[$cnt]!=0) || ($mkm[$cnt]!=0) || ($mkt[$cnt]!=0))) {
           push @month,"<tr><td align=right>".$year_name[$cnt-1]."<\/td><td align=right>".$mkt[$cnt]."<\/td><td align=right>".$mkm[$cnt]."<\/td><td align=right>".$mks[$cnt]."<\/td><\/tr>\n";
      }
      # summarize work each weekday
      if (($cnt<8) && (($dows[$cnt]!=0)||($dows[$cnt]!=0)||($dows[$cnt]!=0))) {
           push @dowli,"<tr><td align=right>".$week_name[($cnt-1)]."<\/td><td align=right>".$dowt[$cnt]."<\/td><td align=right>".$dowm[$cnt]."<\/td><td align=right>".$dows[$cnt]."<\/td><\/tr>\n";
      }
   }



# ------------------------------------------------------------
# all processing is done, lets get something out on the screen 

   print '<p align=center>
              <center><h2>'.$l_work_stats_for.' '.$the_user.'</h2>
              </center></p>
          '."\n";

  # statistics for this year
  print "\n<br>".$std_tableL;
  print '<tr><th colspan=4>'.$text_hcolor.$l_work_this_year.'
             </th>
        </tr>
        <tr><th>'.$l_days.'</th>
            <th>'.$l_hours.'</th>
            <th>'.$l_mins.'</th>
            <th>'.$l_secs.'</th>
        </tr>';
  print '<tr><td align=right>'.$uday.'</td>
             <td align=right>'.$utim.'</td>
             <td align=right>'.$umin.'</td>
             <td align=right>'.$usec.'</td>
         </tr>
         </table><br></b>';

  # monthly statistics
  tblhead4c($l_work_by_month,$l_month,$l_hours,$l_mins,$l_secs,@month);

  # weekly statistics
  tblhead4c($l_work_by_week,$l_week_no,$l_hours,$l_mins,$l_secs,@week);

  # day of week statistics
  tblhead4c($l_work_by_weekdays,$l_day,$l_hours,$l_mins,$l_secs,@dowli);

  # statistics by session
  tblhead5c($l_work_by_sessions,$l_from,$l_to,$l_hours,$l_mins,$l_secs,@list);

}


# ---------------------------------------------------------------------------
# tools

sub tblhead5c() { # write a generic table with 5 columns
  my ($txt,$a1,$a2,$a3,$a4,$a5,@list)=@_;
  print "<br>".$std_tableL;
  print '<tr><th colspan=5>'.$text_hcolor.$txt."<\/font><\/th><\/tr>\n";
  print "<tr><th>".$a1."<\/th><th>".$a2."<\/th><th>".$a3."<\/th><th>".$a4."<\/th><th>".$a5."<\/th></tr>\n";
  foreach $line (@list) {
    print $line;
  }   
  print "<\/table>\n";
}

sub tblhead4c() { # write a generic table with 4 columns
  my ($txt,$a1,$a2,$a3,$a4,@list)=@_;
  print "<br>".$std_tableL;
  print '<tr><th colspan=5>'.$text_hcolor.$txt."<\/font><\/th><\/tr>\n";
  print "<tr><th>".$a1."<\/th><th>".$a2."<\/th><th>".$a3."<\/th><th>".$a4."<\/th></tr>\n";
  foreach $line (@list) {
    print $line;
  }   
  print "<\/table>\n";
}

# take two sets of HHMMSS and adds them together (6 parameters in, 3 out).
sub addtms()
{
	my ($t,$m,$s,$t2,$m2,$s2)=@_;

	$s=$s+$s2;
	if ($s>=60) { $s=$s-60; $m++; }
	$m=$m+$m2;
	if ($m>=60) { $m=$m-60; $t++; }
	$t=$t+$t2;
	return ($t,$m,$s);
}

# -------------------------------------------------------------
# check username and password (we'll need to add crypting here)
sub chk_user()
{
	($unam,$upwd)=@_;
	my ($una,$upw);
	my $the_user="";

	open PWD,"<time.pwd";
	while (<PWD>)
	{
		chomp;
		($una,$gcos_name,$upw)=split /:/;
		if (($una eq $unam) && ($upw eq $upwd))
		{
			$the_user=$gcos_name;
		}
	}
	return ($the_user);
}

# the top of the html document.
sub head()
{
	($menuflag)=@_; # 1==main menu, print copyright.

	print '<html><head><title>Time Manager</title></head>'."\n";
	print '<body bgcolor="'.$screen_bgcolor.'" text="'.$text_color.'">'."\n";
	print '<table width="100%">
		   <tr><td align=left><font size="-1"><b>
			'.$NAME." v".$VERSION.'</b></font></td>
			<td align=right><font size="-1">
				<b>by
				<a href="http://www.ekran.no/">Ekran Design</a>
			</b></font></td></tr></table>';
	print "<hr noshade>\n";
}

sub tail()
{
	($menuflag)=@_; # 0==print logout/menu options  2== print user status.
	my $delta,$gen;	

	print '<br><hr noshade>';

	print '<table width="100%">
            <tr><td align=left>';
	if ($menuflag==0)
	{
		print '<font size="-1"><b>[<a href="'.$cgi_url.'">menu</a>]';
		print '[<a href="'.$cgi_url.'?UACT=logout">logout</a>]</b></font>';
		print '</font>';
	} elsif ($menuflag==2) {
		print '<font size="-1"><b>You ';
		if ($stillon==1)
		{
			$chk=0;
			print "have been signed on since ";
			$myco=0;	
			foreach $line (@dblist)
			{
				($foo,$foo,$timex)=split /;/,$line;
				$myco++;
				if (&ParseDate($timex)) { $gen=$timex; }
			}
			$delta=&DateCalc($gen,&ParseDate($dat),\$err);
        	($foo,$foo,$uw,$ud,$ut,$um,$us)=split /:/,$delta;
			print &UnixDate($gen,$date_format).' (i.e. for ';
			if ($uw ne 0) { print $uw." weeks "; $chk=1;} 
			if (($chk==1) || ($ud ne 0)) { print $ud." days "; } 
			if (($chk==1) || ($ut ne 0)) { print $ut." hours "; } 
			if (($chk==1) || ($um ne 0)) { print $um." minutes "; } 
			if (($chk==1) ||($us ne 0)) { print $us." seconds."; } 
			print ")";
		} else {
			print "are not signed on.";
		}
		print '</b></font>';	
	}
	
	print '</td>
		<td align=right><font size="-1"><b>';
    if ($the_user) { print $the_user.', '; }
	print &UnixDate($dat,$date_format).'<br></font></td>
		</tr></table></body></html>';
	print "\n";
}

# -------------------------------------------------------------
# read datafile and return db+number of entries.

sub read_datafile()
{
	my ($unam)=@_;
	my @dblist;
	
	$entries=0; $id="";

	$dtafile="data/".$unam.".".$this_year.".dta"; # the user database.
	open DTA,"<".$dtafile; # read and buffer datafile
	while ($i=<DTA>)
	{
		chomp $i;
		$entries++;
		push @dblist,$i; # push data onto a list that we'll later use
		($id,$tmp)=split /;/,$i;
	}
	close DTA;
	if ($id) { $next_entry=$id; } else { $next_entry=1; }
	if (substr($tmp,0,3) eq "inn") # add "nempty" if user is still online
	{
		push @dblist,($next_entry+1).";out;nempty";
		$stillon=1;
	} else {
		$stillon=0;
	}
	return (@dblist,$entries);
}

# --------------------------------------------------------------
# read schedulefile if exist, if not, then create it and read it

sub read_sfile()
{
	my ($unam)=@_;
	my $dty=&Date_DaysInYear($this_year);

	$sfile="data/".$unam.".".$this_year.".schedule.dta";
	if (!-f $sfile)
	{
		# did not find a schedule file. Now we'll have to try to create one.
		open DTA,">".$sfile;
		for ($cnt=1; $cnt<=$dty; $cnt++)
		{
			print DTA $cnt.';';
			($y,$m,$d,$foo,$foo,$foo)=&Date_NthDayOfYear($this_year,$cnt);
			$foo=&ParseDate($d." ".$year_name[$m-1]." ".$y);
			if (&Date_IsWorkDay($foo,0)==1)
			{
				print DTA $hours_a_day.";W\n";
			} else {
				print DTA "0;N\n";
			}
		}
		close DTA;
	}

	open DTA,"<".$sfile; # now read the file
	while (<DTA>)
	{
		chomp;
		push @slist,$_;
	}
	close DTA;

	return (@slist);
}

# ------------------------------------------------
# a brief implementation of the norwegian language

# norwegian (and maybe other languages will be supported later.

sub lang_eng()
{
	$l_action_n_spec="You forgot to specify action (signon/signoff/etc..)";
	$l_auth_failed="Username and/or password was not written correctly!";
	$l_bad_seq="You are trying to signon or signoff two times in a row!";
	$l_confirm_action="registered";
	$l_work_stats_for="Work-time statistics for";
	$l_still_online="-- still at work --";
	$l_select_del_title="Select which entry you want to delete";
	$l_select_week_display="Select week to display:<p>";
    $l_select_month_sched='Select which month to reschedule';
	$l_deleted_lines="The following entries was deleted:";
	$l_nodeleted_lines="No lines were deleted.";
 	$l_work_schedule_for="Work schedule for";

	$l_alter_entries="Altering entries.";
	$l_alter_eprealt="can not alter the time/date to a time/date earlier than
						the previous log entry (time/date will overlap.)";
	$l_alter_enextalt="can not alter the time/date to a time/date later than
						the next log entry (time/date will overlap.)";
	$l_alter_futurealt="the time/date you tried to alter to is in the future.";
	$l_alter_illegaldate="the time/date you entered is illegal";
	$l_alter_current="current";
	$l_alter_new="new";
	$l_alter_next="next";
	$l_alter_prev="previous";
	$l_alter_failed="failed";
	$l_alter_noch="no change";
	$l_alter_ch="changed";
	$l_alter_changeent="Alter entry";
	$l_alter_select_entry="Select entry to alter";
	$l_alter_nextillegal="Change failed because next entry is illegal";

	$l_logedout='You have loged out from the system.<br>Have a nice day.';
  	$l_work_by_sessions="Workhours by sessions";
  	$l_work_by_weekdays="Workhours by weekdays";
  	$l_work_by_week="Workhours by week";
  	$l_work_by_month="Workhours by month";
  	$l_work_this_year="Workhours this whole perion (year)";
  	$l_work_stat_week="Workhours for week no.";

	$l_month="month";
  	$l_week_no="week no.";
  	$l_day="day";
  	$l_days="days";
  	$l_from="from";
  	$l_to="to";
  	$l_hours="hours";
  	$l_mins="minutes";
  	$l_secs="seconds";
  	$l_total="total";
  	$l_week_no="Week No.";
  	$l_start="Start";
  	$l_end="End";

  	@year_name=("january","february","march","april","may","june","july","august","september","october","november","december");
	@week_name=("monday","tuesday","wednesday","thursday","friday","saturday","sunday");

}


